#define F_CPU 16000000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <util/delay.h>

#define signum(x) 	(-(x<0) + (x>0))

#define LEVEL 330
#define LEVELDIFF 30
#define LEVEL2 360

#define FORWARD 0
#define LEFT 1
#define RIGHT 2
#define REVERSE 3
#define STOP 4

//pwm max speed
#define MAX_SPEED 1000

////////////////////////////////////////
// data direction and control port for motors input and pwm
#define MOTOR_DD_PORT DDRD
#define MOTOR_PORT PORTB
#define MOTOR_PORT_PIN(x) PORTB##x
//inputs for left motor
#define LEFT_MOTOR_IN1 MOTOR_PORT_PIN(0)
#define LEFT_MOTOR_IN2 MOTOR_PORT_PIN(1)
//inputs for right motor
#define RIGHT_MOTOR_IN1 MOTOR_PORT_PIN(3)
#define RIGHT_MOTOR_IN2 MOTOR_PORT_PIN(2)
//pwm/enable pins
#define LEFT_MOTOR_PWM MOTOR_PORT_PIN(4)
#define RIGHT_MOTOR_PWM MOTOR_PORT_PIN(5)
//////////////////////////////////////////////

//#define SET_DD_IN MOTOR_DD_PORT |= (1 << LEFT_MOTOR_IN1) | (1<<LEFT_MOTOR_IN2) | (1 << RIGHT_MOTOR_IN1) | (1 << RIGHT_MOTOR_IN2) | (1<<LEFT_MOTOR_PWM) | (1<<RIGHT_MOTOR_PWM)

#define MOVE_FORWARD { MOTOR_PORT |= (1<<LEFT_MOTOR_IN1) | (1<<RIGHT_MOTOR_IN1); \
		MOTOR_PORT &= ~((1<<LEFT_MOTOR_IN2) | (1<<RIGHT_MOTOR_IN2)); }

#define MOVE_BACK { MOTOR_PORT |= (1<<LEFT_MOTOR_IN2) | (1<<RIGHT_MOTOR_IN2); \
		MOTOR_PORT &= ~((1<<LEFT_MOTOR_IN1) | (1<<RIGHT_MOTOR_IN1)); }

#define ROTATE_LEFT { MOTOR_PORT |= (1<<LEFT_MOTOR_IN2) | (1<<RIGHT_MOTOR_IN1); \
		MOTOR_PORT &= ~((1<<LEFT_MOTOR_IN1) | (1<<RIGHT_MOTOR_IN2)); }

#define ROTATE_RIGHT { MOTOR_PORT |= (1<<LEFT_MOTOR_IN1) | (1<<RIGHT_MOTOR_IN2); \
		MOTOR_PORT &= ~((1<<LEFT_MOTOR_IN2) | (1<<RIGHT_MOTOR_IN1)); }
////////////////////////////////////////////

int black;
int edge_sensors[3];
char lastdir = STOP;

void init() {

	//enable pull-ups
	PORTA = 0xFF;
	PORTB = 0xFF;
	PORTC = 0xFF;
	PORTD = 0xFF;

	//set data directions
	DDRA = 0x00;
	DDRB = 0xFF;
	DDRC = 0x00;
	DDRD = 0xFF;

	//init timers for PWM; prescaler 8, top = 1000, f = 1khz
	TCCR1A |= (1 << COM1A1) | (1 << COM1B1);
	TCCR1B |= (1 << WGM13) | (1 << CS11);
	ICR1 = MAX_SPEED;
}

int read_adc(char adc_input) {
	ADMUX = adc_input | (1 << REFS0);
	_delay_us(10);
	ADCSRA |= 0x40;
	while ((ADCSRA & 0x10) == 0)
		;
	ADCSRA |= 0x10;
	return ADCW;
}

/*
 speed1 is for left engine, speed2 is for right engine
 */
void speed(int speed1, int speed2) {
	if (abs(speed1) > MAX_SPEED)
		speed1 = MAX_SPEED;
	if (abs(speed2) > MAX_SPEED)
		speed2 = MAX_SPEED;
	OCR1B = abs(speed1);
	OCR1A = abs(speed2);
}

void move_forward(int speed1, int speed2) {
	MOVE_FORWARD;
	speed(speed1, speed2);
	lastdir = FORWARD;
}

void move_back(int speed1, int speed2) {
	MOVE_BACK;
	speed(speed1, speed2);
	lastdir = REVERSE;
}

//dir = 0 rotate to right
//else rotate to left
void rotate(int s1, int s2, char dir) {
	if (dir == 0) {
		ROTATE_RIGHT;
		speed(s1, s2);
		lastdir = RIGHT;
	} else {
		ROTATE_LEFT;
		speed(s1, s2);
		lastdir = LEFT;
	}
}

void stop() {
	speed(0, 0);
	lastdir = STOP;
}

void move(int speed1, int speed2) {
	if ((speed1 >= 0) && (speed2 >= 0))
		move_forward(speed1, speed2);
	else if ((speed1 <= 0) && (speed2 >= 0))
		rotate(speed1, speed2, 1);
	else if ((speed1 >= 0) && (speed2 <= 0))
		rotate(speed1, speed2, 0);
	else
		move_back(speed1, speed2);
}



char get_sensors() {

	char left,center,right;

	left = center = right = 0;

	if (bit_is_set(PINA,0))
		left = 1;
	if (bit_is_set(PINA,1))
		center = 2;
	if (bit_is_set(PINA,2))
		right = 4;
	return (left + center + right);
}


int main() {


	char val = 0;

	init(); //setup basic stuff like data direction for ports, adc, timers


	for (;;) {

		val = get_sensors();
		if (val == 0) //no line
			move(500, 500);
		else if(val == 1) //line on the left sensor
			move(-600,1000);
		else if(val == 2) //line on the center
			move(700,700);
		else if(val == 3) //left + center
			move(-200,1000);
		else if(val == 4) // right
			move(1000,-600);
		else if (val == 5) //  right + left??
			move(500,500);
		else if (val == 6) // center + right
			move(1000,-200);
		else if (val == 7) // all??
			move(700,-700);




	}
	return 0;
}
